/**
 * Custom converter for the class name
 *
 * @callback classNameConverter
 * @param {string} className - The class name.
 */

/**
 * Provides a wrapper around an API similar to [classnames#bind](https://github.com/JedWatson/classnames#alternate-bind-version-for-css-modules)
 * that also supports converting class names before they are looked up in
 * the `classNameMap`.
 *
 * @param {Record<string, string>} classNameMap - A map of class names
 * @param {classNameConverter} classNameConverter - A callback function to convert the class name before being looked up in the class name map
 *
 * @returns {function(...[*]): string} - Returns an API equivalent to [classnames](https://github.com/JedWatson/classnames#usage).
 */
function createClassNames(classNameMap, classNameConverter) {
  if (classNameMap === void 0) {
    classNameMap = {};
  }

  if (classNameConverter === void 0) {
    classNameConverter = function classNameConverter(className) {
      return className;
    };
  }

  return function internalClassNames() {
    for (var _len = arguments.length, classNamesOrObjects = new Array(_len), _key = 0; _key < _len; _key++) {
      classNamesOrObjects[_key] = arguments[_key];
    }

    return classNamesOrObjects.filter(function (item) {
      return item;
    }).map(function (item) {
      function convert(className) {
        var convertedClassName = classNameConverter(className);

        if (classNameMap[convertedClassName]) {
          return classNameMap[convertedClassName];
        }

        if (className.trim().indexOf(' ') > -1) {
          return internalClassNames.apply(void 0, className.trim().split(' '));
        }

        return className;
      }

      var itemType = typeof item;

      if (itemType === 'string' || itemType === 'number') {
        return convert("" + item);
      }

      if (Array.isArray(item)) {
        return internalClassNames.apply(void 0, item);
      }

      if (itemType === 'object') {
        return internalClassNames(Object.keys(item).filter(function (key) {
          return item[key];
        }));
      }

      return null;
    }).filter(function (item) {
      return item !== null;
    }).join(' ').trim();
  };
}

export default createClassNames;